@@ -94,27 +94,11 @@ class Application(object):
             self.id = configuration.id
         else:
             self.id = __name__
-        #bw 0.9.3
-        self.modules = self.registry
         
         self.__name__ = u""
         self.__parent__ = None
         self.__acl__ = []
 
-        # 0.9.4 - moved to configuration
-        #self.id = u""
-        #self.title = u""
-        #self.description = u""
-    
-        # data pool
-        # 0.9.4 - moved to configuration
-        #self.fulltextIndex = False
-        #self.autocommit = True
-        #self.useCache = True
-        #self.frontendCodepage = "utf-8"
-        #self.workflowEnabled = False
-        #self.groups = []
-        #self.categories = []
         # development
         self.debug = False
         self.reloadExtensions = False
@@ -127,6 +111,7 @@ class Application(object):
         self._structure = PoolStructure()
         self._dbpool = None
         log = logging.getLogger(self.id)
+        
         log.debug("Initialize %s", repr(configuration))
         self.Signal("init", configuration=configuration)
 
@@ -211,7 +196,12 @@ class Application(object):
     # Properties -----------------------------------------------------------
 
     def root(self, name=""):
-        """ returns root object     """
+        """ 
+        Events:
+        - loadRoot(root)
+
+        returns root object
+        """
         return self._GetRootObj(name)
 
     def obj(self, id, rootname = "", **kw):
@@ -251,6 +241,9 @@ class Application(object):
         """
         Returns the data root object. If name is empty the default root is returned.
         
+        Events:
+        - loadRoot(root)
+
         returns root object
         """
         return self._GetRootObj(name)
@@ -289,13 +282,6 @@ class Application(object):
         return self._GetWfObj(wfProcID, contextObject or _GlobalObject())
 
 
-    def NewDBConnection(self):
-        """
-        Creates a new database connection. This one is independent from caching and 
-        connections used internally.  
-        """
-        return self._GetConnectionObj()
-        
     # Data Pool ------------------------------------------------------------------------------
 
     def GetDB(self):
@@ -328,17 +314,6 @@ class Application(object):
         return r
 
 
-    def GetCountEntries(self):
-        """
-        Get the total number of entries in the data pool.
-        
-        returns number
-        """
-        db = self.db
-        c = db.GetCountEntries()
-        return c
-
-
     def TestDB(self):
         """
         Test database connection for errors. Returns state and list with errors.
@@ -357,17 +332,19 @@ class Application(object):
             return False, e
 
 
-    def ConvertID(self, id):
+    def NewConnection(self):
         """
-        Convert id to number.
+        Creates a new database connection. Works before startup and application setup.
+        """
+        return self._GetConnection()
         
-        returns number
-        """
-        try:
-            return int(id)
-        except:
-            return -1
 
+    def NewDBApi(self):
+        """
+        Creates a raw database connection (dbapi). Works before startup and application setup.
+        """
+        return self._GetDBApi()
+        
 
     def GetVersion(self):
         """ """
@@ -378,6 +355,25 @@ class Application(object):
         return __version__ == __version__
 
 
+    # to be removed in future versions -------------------------------------------
+
+    def NewDBConnection(self):
+        return self.NewConnection()
+    
+    def ConvertID(self, id):
+        try:
+            return int(id)
+        except:
+            return -1
+
+    def GetCountEntries(self):
+        db = self.db
+        c = db.GetCountEntries()
+        return c
+
+
+
+
 class Registration(object):
 
     def Register(self, module, **kw):
@@ -431,6 +427,13 @@ class Registration(object):
         # register module itself
         if hasattr(conf, "unlock"):
             conf.unlock()
+        
+        # events
+        if conf.get("events"):
+            for e in conf.events:
+                log.debug('Register Event: %s for %s', str(e.event), str(e.callback))
+                self.ListenEvent(e.event, e.callback)
+        
         if iface == IRootConf:
             self.registry.registerUtility(conf, provided=IRootConf, name=conf.id)
             if conf.default or not self._defaultRoot:
@@ -473,11 +476,6 @@ class Registration(object):
             return True
         
         elif iface == IModuleConf:
-            # events
-            if conf.events:
-                for e in conf.events:
-                    log.debug('Register Event: %s for %s', str(e.event), str(e.callback))
-                    self.RegisterEvent(e.event, e.callback)
             # modules
             if conf.modules:
                 for m in conf.modules:
@@ -607,7 +605,7 @@ class Registration(object):
                                 name=view.name,
                                 view=view.view or viewmod.view,
                                 context=view.context or viewmod.context,
-                                renderer=view.renderer,
+                                renderer=view.renderer or viewmod.renderer,
                                 permission=view.permission or viewmod.permission,
                                 containment=view.containment or viewmod.containment,
                                 custom_predicates=view.custom_predicates or viewmod.custom_predicates or (self.AppViewPredicate,),
@@ -1064,7 +1062,7 @@ class AppFactory:
     Internal class for dynamic object creation and caching.
 
     Requires:
-    - Workflow.Workflow
+    - Application
     """
 
     def _GetDataPoolObj(self):
@@ -1083,7 +1081,7 @@ class AppFactory:
         cTag = self.dbConfiguration.connection
         if cTag:
             connObj = GetClassRef(cTag, self.reloadExtensions, True, None)
-            connObj = connObj(config=self.dbConfiguration)
+            connObj = connObj(config=self.dbConfiguration, connectNow=False)
         else:
             connObj = None
 
@@ -1100,15 +1098,21 @@ class AppFactory:
         return dbObj
 
     
-    def _GetConnectionObj(self):
+    def _GetConnection(self):
         """
-        creates a new database connection object
+        Creates a new database connection. Works before startup and application setup.
         """
+        if self._dbpool:
+            try:
+                return self._dbpool.usedconnection
+            except:
+                pass
+                
         cTag = self.dbConfiguration.connection
         poolTag = self.dbConfiguration.context
         if cTag:
             connObj = GetClassRef(cTag, self.reloadExtensions, True, None)
-            connObj = connObj(config=self.dbConfiguration)
+            connObj = connObj(config=self.dbConfiguration, connectNow=False)
             return connObj
         if self._dbpool:
             return self._dbpool.CreateConnection(self.dbConfiguration)
@@ -1122,7 +1126,17 @@ class AppFactory:
         elif poolTag.lower() == "mysql":
             poolTag = "nive.utils.dataPool2.mySqlPool.MySql"
         dbObj = GetClassRef(poolTag, self.reloadExtensions, True, None)
-        return dbObj.defaultConnection(self.dbConfiguration)
+        return dbObj._DefaultConnection(config=self.dbConfiguration, connectNow=False)
+
+
+    def _GetDBApi(self):
+        """
+        Creates a new and raw database connection. Works before startup and application setup.
+        """
+        conn = self._GetConnection()
+        if conn:
+            return conn.PrivateConnection()
+        return None
 
 
     def _GetRootObj(self, rootConf):
@@ -1144,6 +1158,7 @@ class AppFactory:
             rootObj = rootObj(name, self, rootConf)
             if rootObj and useCache:
                 setattr(self, cachename, rootObj)
+        self.Signal("loadRoot", rootObj)
         return rootObj
 
 
