@@ -18,54 +18,85 @@
 __doc__ = """
 Nive cms toolbox and editor view layer
 ----------------------------------------
+The view configuration includes a list of assets (javascript and css) needed to render the toolbox
+and editor widgets. These are included in the design/main template in edit mode through `cmsview.Assets()`.
+Asset definitions use a identifier and an asset path like: ::
+
+    ('jquery.js', 'nive.cms.cmsview:static/mods/jquery.min.js'),
+    ('toolbox.css', 'nive.cms.cmsview:static/toolbox/toolbox.css'),
+
+If for example jquery is already included in the main page Assets() can be told to ignore certain
+entries: ::
+
+    cmsview.Assets(ignore=["jquery.js"])  
+
 """
 try:
     from cStringIO import StringIO
 except:
     from StringIO import StringIO
+
+from pyramid.response import Response
+from pyramid.renderers import get_renderer, render, render_to_response
+
     
 from nive.i18n import _, translator
-from nive.definitions import ViewModuleConf, ViewConf, WidgetConf
-from nive.definitions import IContainer, IApplication, IPortal, IPage, IObject, IRoot, IToolboxWidgetConf, IEditorWidgetConf
-from nive.definitions import IToolboxWidgetConf, IEditorWidgetConf, ICMSRoot, IColumn
-from nive.cms.design import view as design 
+from nive.definitions import ViewModuleConf, ViewConf, WidgetConf, FieldConf
+from nive.definitions import IContainer, IApplication, IPortal, IPage, IObject, IRoot
+from nive.definitions import IToolboxWidgetConf, IEditorWidgetConf, IViewModuleConf, ICMSRoot, IColumn
 from nive.utils.utils import SortConfigurationList
+from nive.helper import ResolveName
+
+from nive.forms import ObjectForm
+from nive.views import BaseView
+
+from nive.cms.cmsview import cutcopy
+from nive.cms.cmsview import sort
+
 
 
 # view module definition ------------------------------------------------------------------
 
 #@nive_module
 configuration = ViewModuleConf(
-    id = "cmsview",
+    id = "editor",
     name = u"CMS Editor",
     static = "nive.cms.cmsview:static",
     templates = "nive.cms.cmsview:",
     permission = "read",
     context = IObject,
-    containment = ICMSRoot,  #"nive.cms.cmsview.cmsroot.cmsroot",
-    view = "nive.cms.cmsview.view.Editor"
+    containment = ICMSRoot,  
+    view = "nive.cms.cmsview.view.Editor",
+    assets = [
+        # jquery and jquery-ui
+        ('jquery.js', 'nive.cms.cmsview:static/mods/jquery.min.js'),
+        ('jquery-ui.js', 'nive.cms.cmsview:static/mods/ui/jquery-ui-1.8.24.custom.min.js'),
+        # nive specific
+        ('cmseditor.js', 'nive.cms.cmsview:static/cmseditor.js'),
+        ('toolbox.css', 'nive.cms.cmsview:static/toolbox/toolbox.css'),
+        ('overlay.css', 'nive.cms.cmsview:static/overlay/overlay.css'),
+        ('cmseditor.css', 'nive.cms.cmsview:static/cmseditor.css'),
+    ],
+    description=__doc__
 )
-# views -----------------------------------------------------------------------------
-# shortcuts
-t = configuration.templates 
 
+# views -----------------------------------------------------------------------------
+# copy and sort extension views are imported from their files and added at the end of the list
+
+t = configuration.templates 
 configuration.views = [
-    ViewConf(name = "editor",     attr = "editor",  context=IContainer,   permission="view", containment=IApplication),
-    ViewConf(name = "exiteditor", attr = "exit",    context=IContainer,   permission="view", containment=IApplication),
-    ViewConf(name = "exiteditor", attr = "exitapp", context=IApplication, permission="view", containment=IPortal),
+    ViewConf(name="editor",     attr="editor",  context=IContainer,   permission="read", containment=IApplication),
+    ViewConf(name="exiteditor", attr="exit",    context=IContainer,   permission="view", containment=IApplication),
+    ViewConf(name="exiteditor", attr="exitapp", context=IApplication, permission="view", containment=IPortal),
     
-    ViewConf(id="rootview", name = "",     attr = "view", context = ICMSRoot, containment=IApplication),
-    ViewConf(id="objview",  name = "",     attr = "view", context = IPage),
+    #ViewConf(id="rootview", name = "",     attr = "view", context = ICMSRoot, containment=IApplication),
+    #ViewConf(id="objview",  name = "",     attr = "view", context = IPage),
     
     # object
-    ViewConf(name = "edit", attr = "edit", renderer = t+"edit.pt", permission="edit"),
-    ViewConf(name = "meta", attr = "meta", renderer = t+"meta.pt"),
-    ViewConf(name ="delfile",attr= "delfile", permission="delete"),
+    ViewConf(name="edit", attr="edit", renderer=t+"edit.pt", permission="edit"),
+    ViewConf(name="meta", attr="meta", renderer=t+"meta.pt"),
+    ViewConf(name="delfile", attr="delfile", permission="delete"),
                 
-    # cut, copy
-    ViewConf(name = "cut",  attr = "cut",  context = IContainer, permission="edit"),
-    ViewConf(name = "copy", attr = "copy", context = IContainer, permission="edit"),
-    
     # widgets
     ViewConf(name = "elementListWidget", attr = "elementListWidget", context = IContainer, permission="edit"),
     ViewConf(name = "elementAddWidget",  attr = "elementAddWidget",  context = IObject, permission = "add"),
@@ -76,24 +107,12 @@ configuration.views = [
     ViewConf(name = "add",       attr = "add",    context = IContainer, renderer = t+"add.pt", permission="add"),
     ViewConf(name = "delete",    attr = "delete", context = IContainer, renderer = t+"delete.pt", permission = "delete"),
     
-    # sort
-    ViewConf(name = "sortpages", attr = "sortpages", context = IPage, renderer = t+"sort.pt", permission="edit"),
-    ViewConf(name = "sortpages", attr = "sortpages", context = IRoot, renderer = t+"sort.pt", permission="edit"),
-    ViewConf(name="sortelements",attr="sortelements",context = IContainer, renderer = t+"sort.pt", permission="edit"),
-    ViewConf(name = "moveup",    attr = "moveup",    context = IContainer, permission="edit"),
-    ViewConf(name = "movedown",  attr = "movedown",  context = IContainer, permission="edit"),
-    ViewConf(name = "movetop",   attr = "movetop",   context = IContainer, permission="edit"),
-    ViewConf(name = "movebottom",attr = "movebottom",context = IContainer, permission="edit"),
-    
-    # paste
-    ViewConf(name = "paste", attr = "paste", context = IContainer, permission = "add"),
-    
     # widgets
     ViewConf(name = "addpageWidget",  attr = "tmpl", renderer = t+"widgets/widget_addpage.pt",    context = IContainer, permission="add"),
     ViewConf(name = "editpageWidget", attr = "tmpl", renderer = t+"widgets/widget_editpage.pt",   context = IContainer, permission="edit"),
     ViewConf(name = "subpagesWidget", attr = "tmpl", renderer = t+"widgets/widget_subpages.pt",   context = IContainer),
     ViewConf(name = "settingsWidget", attr = "tmpl", renderer = t+"widgets/widget_settings.pt",   context = IContainer)
-]
+] + sort.views + cutcopy.views
 
 
 # toolbox and editor widgets ----------------------------------------------------------------------------------
@@ -114,17 +133,8 @@ configuration.widgets = [
         
 # view implementation ------------------------------------------------------------------
         
-from nive.forms import ObjectForm
-from nive.views import BaseView
 
-from nive.components.extensions.sort import SortView
-from nive.components.extensions.cutcopy import CopyView
-
-from pyramid.response import Response
-from pyramid.renderers import get_renderer, render, render_to_response
-
-
-class Editor(BaseView, CopyView, SortView):
+class Editor(BaseView, cutcopy.CopyView, sort.SortView):
 
     def __init__(self, context, request):
         BaseView.__init__(self, context, request)
@@ -144,7 +154,41 @@ class Editor(BaseView, CopyView, SortView):
             confs.append(w)
         return SortConfigurationList(confs, u"sort")
         
+    def Assets(self, assets=None, ignore=None):
+        """
+        Renders a list of static ressources as html <script> and <link>.
+        If assets is None the list of assets is looked up in the 'cmsview'-configuration.
+        Asset definitions use a identifier and an asset path like: ::
+        
+            ('jquery.js', 'nive.cms.cmsview:static/mods/jquery.min.js'),
+            ('toolbox.css', 'nive.cms.cmsview:static/toolbox/toolbox.css'),
+            ('jquery-ui.js', 'http://example.com/static/mods/jquery.min.js'),
+        
+        If for example jquery is already included in the main page Assets() can be told to ignore certain
+        entries: ::
+        
+            cmsview.Assets(ignore=["jquery.js"])  
+        
+        """
+        if assets==None:
+            app = self.context.app
+            conf = app.QueryConfByName(IViewModuleConf, "editor")
+            if not conf:
+                return u""
+            assets = conf.assets
 
+        if not assets:
+            return u""
+        
+        if ignore==None:
+            ignore = []
+
+        js_links = [r[1] if r[1].startswith(u"http://") else self.StaticUrl(r[1]) for r in filter(lambda v: v[0] not in ignore and v[1].endswith(u".js"), assets)]
+        css_links = [r[1] if r[1].startswith(u"http://") else self.StaticUrl(r[1]) for r in filter(lambda v: v[0] not in ignore and v[1].endswith(u".css"), assets)]
+        js_tags = [u'<script src="%s" type="text/javascript"></script>' % link for link in js_links]
+        css_tags = [u'<link href="%s" rel="stylesheet" type="text/css" media="all"/>' % link for link in css_links]
+        return (u"\r\n").join(js_tags + css_tags)
+        
     # macros ------------------------------------------------------------------------ 
 
     def cmsIndex_tmpl(self):
@@ -178,6 +222,23 @@ class Editor(BaseView, CopyView, SortView):
     
     # cms editor interface elements -------------------------------------------------
     
+    def cms(self, obj=None):
+        """
+        Renders required widgets and toolbar
+        
+        Calls
+        - self.cmsToolbox() 
+        - self.cmsEditorBlocks()
+        - self.cmsEditorBlocks() for each column
+        """
+        if not obj:
+            obj=self.context
+        html = [self.cmsToolbox(obj), self.cmsEditorBlocks(obj)]
+        for column in obj.app.configuration.columns:
+            html.append(self.cmsEditorBlocks(obj.page.GetColumn(column)))
+        return u"".join(html)
+
+    
     def cmsToolbox(self, obj, elements=None):
         """
         nive toolbox widget.
@@ -276,18 +337,20 @@ class Editor(BaseView, CopyView, SortView):
         return data
 
 
-    def editBlockColumn(self, page=None, column=None, name=None):
+    def editBlockColumn(self, name=None, page=None, column=None):
         """
         Edit bar for columns
-        if column is None current context is used
+        if column is None the current context is used
         """
+        if not page:
+            page = self.context.page
         if not column:
             column=self.context
             if not IColumn.providedBy(column):
-                column = None
-        if page == None and name == None and column != None:
+                if name:
+                    column = page.GetColumn(name)
+        if column != None:
             name = column.meta.get("title")
-            page = column.GetPage()
         return render("widgets/editblock_column.pt", 
                       {u"column":column, u"page": page, u"name": name, u"view":self}, 
                       request=self.request)
@@ -369,7 +432,7 @@ class Editor(BaseView, CopyView, SortView):
                 title = u"<img src='%s%s.png' align='top'/> %s" % (static, el.GetTypeID(), t)
                 blocks.write(elHtml % {u"title": title, u"options": self.editBlockList(obj=el, showCCP=True)})
         if not len(elements):
-            blocks.write(localizer(_(u"<i>empty</i>")))
+            blocks.write(localizer(_(u"<p><i>empty</i></p>")))
         data = html % {u"blocks": blocks.getvalue(), u"id": str(obj.GetID()), u"title": localizer(_(u"Page elements"))}
         if addResponse:
             r = Response(content_type="text/html", conditional_response=True)
@@ -428,17 +491,29 @@ class Editor(BaseView, CopyView, SortView):
             return u""
         html = StringIO()
         for page in parents:
+            title = page.GetTitle() or page.GetTypeName()
             if not link:
-                html.write(u"""<span>%s</span> &gt; """ % (page.GetTitle()))
+                html.write(u"""<span>%s</span> &gt; """ % (title))
             else:
-                html.write(u"""<a href="%s" class="nivecms">%s</a> &gt; """ % (self.PageUrl(page), page.GetTitle()))
+                html.write(u"""<a href="%s" class="nivecms">%s</a> &gt; """ % (self.PageUrl(page), title))
+        title = base.GetTitle() or base.GetTypeName()
         if not link:
-            html.write(u"""<span>%s</span>""" % (base.GetTitle()))
+            html.write(u"""<span>%s</span>""" % (title))
         else:
-            html.write(u"""<a href="%s" class="nivecms">%s</a>""" % (self.PageUrl(base), base.GetTitle()))
+            html.write(u"""<a href="%s" class="nivecms">%s</a>""" % (self.PageUrl(base), title))
         return html.getvalue()
     
     
+    def insertMessages(self):
+        messages = self.request.session.pop_flash("")
+        if not messages:
+            return u""
+        html = u"""<div class="alert"> <a href="#" class="close" data-dismiss="alert">&times;</a> <ul><li>%s</li></ul></div>"""
+        try:
+            return html % (u"</li><li>".join(messages))
+        except:
+            return u""
+        
     def insertPageWidgets(self):
         return self.insertToolboxWidgets(self.context.GetPage())
         
@@ -491,14 +566,27 @@ class Editor(BaseView, CopyView, SortView):
 
     # template rendering -----------------------------------------------------------------------
     
-    def view(self):
-        #if self..Allowed(u"edit", self.context):
-        #    vars["cmsview"] = cmsview.CMS(self.context, self.request)
-        d = design.Design(self.context, self.request)
-        html = d.view(self)
-        self.NoCache(self.request.response, user=self.User())
-        return html
+    @property
+    def editorview(self):
+        return self
     
+    @property
+    def designview(self):
+        """
+        Tries to load the editor view class. If none is registered the function 
+        will simply return None. Otherwise the editor view class instance with 
+        context and request set.
+        """
+        if hasattr(self, "_c_design"):
+            return self._c_design
+        module = self.context.app.QueryConfByName(IViewModuleConf, "design")
+        if not module:
+            return None
+        cls =  ResolveName(module.view)
+        design = cls(self.context, self.request)
+        self._c_design = design
+        return design
+        
     
     def add(self):
         typeID = self.GetFormValue("pool_type")
@@ -508,7 +596,7 @@ class Editor(BaseView, CopyView, SortView):
         form.Setup(subset="create", addTypeField=True)
         form.use_ajax = True
         head = form.HTMLHead()
-        result, data, action = form.Process(defaultAction="default", redirect_success="page_url")
+        result, data, action = form.Process(redirectSuccess="page_url")
         return {u"content": data, u"result": result, u"cmsview": self, u"showAddLinks": False, u"head": head}
 
     
@@ -527,6 +615,11 @@ class Editor(BaseView, CopyView, SortView):
             result[u"objToDelete"] = obj
             return result
         result[u"result"] = self.context.Delete(id, user=self.User(), obj=obj)
+        if result[u"result"]:
+            result[u"msgs"] = [_(u"OK. Deleted.")]
+        self.Relocate("""<script type="text/javascript">window.parent.close("%s");</script>""" % (self.PageUrl()), 
+                      [_(u"OK. Deleted.")], 
+                      raiseException=True)
         return result
     
     
@@ -535,7 +628,7 @@ class Editor(BaseView, CopyView, SortView):
         form.use_ajax = True
         form.Setup(subset="edit")
         head = form.HTMLHead()
-        result, data, action = form.Process(defaultAction="defaultEdit", redirect_success="page_url")
+        result, data, action = form.Process(redirectSuccess="page_url")
         return {u"content": data, u"result": result, u"cmsview":self, u"head": head}
 
 
