@@ -1,6 +1,5 @@
 #----------------------------------------------------------------------
-# Nive cms
-# Copyright (C) 2012  Arndt Droullier, DV Electric, info@dvelectric.com
+# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,123 +25,21 @@ mailing options.
 import types, base64, random, string
 from time import time
 import uuid
+import json
 
-from nive.definitions import RootConf, Conf, StagUser
-from nive.security import User
+from nive.definitions import RootConf, Conf, StagUser, IUser
+from nive.definitions import Interface, implements
+from nive.security import User, AdminUser, IAdminUser, UserFound
 from nive.components.objects.base import RootBase
 from nive.i18n import _
 
 
-class Unauthorized(Exception):
+class root(RootBase):
     """
-    failed login
     """
-
-
-import thread
-
-class UserCache(object):
-    """
-    User caching support. Caches users including data as attributes. 
     
-    Options: ::
-
-        useCache = enable or disable caching
-        cacheTypes = a list of pool_types to be cached. not matching types are not cached
-        expires = objs are reloaded or purged after this many seconds. 0 = never expires 
-    """
-    useCache = True
-    cacheTypes = (u"user",)
-    expires = 0 
-
-    def Cache(self, obj, id):
-        """
-        """
-        if not self.useCache:
-            return 
-        try:
-            t = obj.GetTypeID()
-            if self.cacheTypes and not t in self.cacheTypes:
-                return
-        except:
-            if self.cacheTypes:
-                return 
-        try:
-            lock = thread.allocate_lock()
-            lock.acquire(1)
-            setattr(self, self._Cachename(id), (obj, time()))
-            if lock.locked():
-                lock.release()
-        except:
-            if lock and lock.locked():
-                lock.release()
-
-    def GetFromCache(self, id):
-        """
-        returns the cached object
-        """
-        if not self.useCache:
-            return None
-        n = self._Cachename(id)
-        try:
-            lock = thread.allocate_lock()
-            lock.acquire(1)
-            if hasattr(self, n):
-                o = getattr(self, n)
-                if lock.locked():
-                    lock.release()
-                return o[0]
-        except:
-            if lock and lock.locked():
-                lock.release()
-        return None
-
-    def GetAllFromCache(self):
-        """
-        returns all cached objects
-        """
-        objs = []
-        try:
-            lock = thread.allocate_lock()
-            lock.acquire(1)
-            for v in self.__dict__.keys():
-                if v[:5] == "__c__":
-                    objs.append(getattr(self, v)[0])
-            if lock.locked():
-                lock.release()
-        except:
-            if lock and lock.locked():
-                lock.release()
-        return objs
-
-    def RemoveCache(self, id):
-        """
-        """
-        if not self.useCache:
-            return 
-        try:
-            lock = thread.allocate_lock()
-            lock.acquire(1)
-            try:
-                delattr(self, self._Cachename(id))
-            except:
-                pass
-            if lock.locked():
-                lock.release()
-        except:
-            if lock and lock.locked():
-                lock.release()
-
-    def _Cachename(self, id):
-        return "__c__" + str(hash(str(id)))
-
-
-
-
-
-class root(UserCache, RootBase):
-    """
-    """
+    # field used as unique user identity in sessions and acache
+    identityField = u"name"
     
     # User account handling ------------------------------------------------------------------------------------------------------
 
@@ -169,9 +66,16 @@ class root(UserCache, RootBase):
             return None, report
 
         # check user with name exists
-        if self.LookupUser(name=name, activeOnly=0):
+        user = self.GetUserByName(name, activeOnly=0)
+        if user:
             report.append(_(u"Username '${name}' already in use. Please choose a different name.", mapping={u"name":name}))
             return None, report
+        email = data.get("email")
+        if email and self.app.configuration.get("loginByEmail"):
+            user = self.GetUserByMail(email, activeOnly=0)
+            if user:
+                report.append(_(u"Email '${name}' already in use. ", mapping={'name':email}))
+                return None, report
         
         if generatePW:
             pw = self.GeneratePassword()
@@ -198,7 +102,7 @@ class root(UserCache, RootBase):
             body = mail(user=obj, **kw)
             tool = app.GetTool("sendMail")
             try:
-                result, value = tool(body=body, title=title, recvids=[name], force=1)
+                result, value = tool(body=body, title=title, recvids=[str(obj)], force=1)
                 if not result:
                     report.append(_(u"The email could not be sent."))
                     return None, report
@@ -250,7 +154,7 @@ class root(UserCache, RootBase):
         if not obj:
             obj = self.GetUserByName(email)
             if not obj:
-                report.append(_(u"No account found with that email address. Please try again."))
+                report.append(_(u"No matching account found. Please try again."))
                 return False, report
 
         email = obj.data.get("email")
@@ -287,35 +191,6 @@ class root(UserCache, RootBase):
         return True, report
 
 
-    def GenerateID(self, length=20, repl="-"):
-        # generates a id
-        return str(uuid.uuid4()).replace(repl,"")[:length]
-        
-
-    def DeleteUser(self, name):
-        """
-        returns status and report list
-        """
-        report = []
-        # check email exists
-        if name == "" or not name:
-            report.append(_(u"Invalid username."))
-            return False, report
-
-        user = self.LookupUser(name=name, activeOnly=0)
-        if not user:
-            report.append(_(u"Invalid username."))
-            return False, report
-
-        self.RemoveCache(name)
-        if not self.Delete(user.GetID(), obj=user, user=user):
-            report.append(_(u"Sorry. An error occured."))
-            return False, report
-
-        report.append(_(u"User deleted."))
-        return True, report
-
-
     # Login/logout and user sessions ------------------------------------------------------------------------------------------------------
 
     def Login(self, name, password, raiseUnauthorized = 1):
@@ -329,34 +204,42 @@ class root(UserCache, RootBase):
         if not user:
             if raiseUnauthorized:
                 raise Unauthorized, "Login failed"
-            report.append(_(u"Login failed. Please check your username and password."))
+            report.append(_(u"Sign in failed. Please check your username and password."))
             return None, report
             
         if not user.Authenticate(password):
             if raiseUnauthorized:
                 raise Unauthorized, "Login failed"
-            report.append(_(u"Login failed. Please check your username and password."))
+            report.append(_(u"Sign in failed. Please check your username and password."))
             return None, report
 
         # call user
         user.Login()
-        report.append(_(u"Logged in."))
+        report.append(_(u"You are now signed in."))
         return user, report
 
 
-    def Logout(self, name):
+    def Logout(self, ident):
         """
         Logout and delete session data
         """
-        user = self.GetUserByName(name)
+        user = self.GetUser(ident)
         if not user:
             return False
-        user.Logout()
+        if not IUser.providedBy(user):
+            user = self.LookupUser(id=user.id)
+        if user:
+            user.Logout()
         return True
 
 
     # Password, activationID ------------------------------------------------------------------------------------------------------
 
+    def GenerateID(self, length=20, repl="-"):
+        # generates a id
+        return str(uuid.uuid4()).replace(repl,"")[:length]
+        
+
     def GeneratePassword(self, mincount=5, maxcount=5):
         # generates a password
         vowels = u"aeiou0123456789#*"
@@ -376,6 +259,175 @@ class root(UserCache, RootBase):
         return password
 
 
+    # User ------------------------------------------------------------------------------------------------------
+
+    def GetUser(self, ident, activeOnly=1):
+        """
+        Lookup user by *user identity* as used in session cookies for example
+        
+        events: 
+        - getuser(ident, activeOnly)
+        - loaduser(user)
+        """
+        try:
+            self.Signal("getuser", ident=ident, activeOnly=activeOnly)
+        except UserFound, user:
+            return user.user
+        user = self.LookupUser(ident=ident, activeOnly=activeOnly)
+        if user:
+            self.Signal("loaduser", user=user)
+        return user
+    
+
+    def GetUserByName(self, name, activeOnly=1):
+        """ """
+        return self.LookupUser(name=name, activeOnly=activeOnly)
+
+
+    def GetUserByMail(self, email, activeOnly=1):
+        """ """
+        return self.LookupUser(email=email, activeOnly=activeOnly)
+
+
+    def GetUserByID(self, id, activeOnly=1):
+        """ """
+        return self.LookupUser(id=id, activeOnly=activeOnly)
+
+
+    def LookupUser(self, id=None, ident=None, name=None, email=None, activeOnly=1, reloadFromDB=0):
+        """ 
+        reloadFromDB deprecated. will be removed in the future
+        """
+        if not id:
+            loginByEmail = self.app.configuration.get("loginByEmail")
+            # lookup adminuser
+            admin = self.app.configuration.get("admin")
+            if admin:
+                if ident and ident == admin.get(self.identityField):
+                    return AdminUser(admin, admin[self.identityField])
+                elif name and name == admin["name"]:
+                    return AdminUser(admin, admin.get(self.identityField))
+                elif loginByEmail and email and email == admin["email"]: 
+                    return AdminUser(admin, admin.get(self.identityField))
+            # lookup id for name, email or ident
+            param = {}
+            if activeOnly:
+                param[u"pool_state"] = 1
+            if name:
+                param[u"name"] = name
+            elif email:
+                param[u"email"] = email
+            elif ident:
+                if not self.identityField:
+                    raise ValueError, "user identity filed not set"
+                param[self.identityField] = ident
+                
+            user = self.Select(pool_type=u"user", parameter=param, fields=[u"id"], max=2)
+            
+            # check multiple identity fields
+            if len(user)==0 and loginByEmail:
+                if name:
+                    del param[u"name"]
+                    param[u"email"] = name
+                    user = self.Select(pool_type=u"user", parameter=param, fields=[u"id"], max=2)
+                elif email:
+                    del param[u"email"]
+                    param[u"name"] = email
+                    user = self.Select(pool_type=u"user", parameter=param, fields=[u"id"], max=2)
+                else:
+                    return None
+            
+            if len(user)!=1:
+                return None
+            id = user[0][0]
+            
+        user = self.GetObj(id)
+        if not user or (activeOnly and not user.meta.get("pool_state")==1):
+            return None
+        return user
+     
+
+    def GetUsers(self, **kw):
+        """
+        """
+        return self.SearchType(u"user", {u"pool_state":1}, [u"id",u"title",u"name",u"groups",u"lastlogin"])
+
+
+    def GetUserInfos(self, userids, fields=None, activeOnly=True):
+        """
+        """
+        param = {self.identityField:userids}
+        if activeOnly:
+            param[u"pool_state"] = 1
+        if not fields:
+            fields = [u"id", u"name", u"email", u"title", u"groups", self.identityField]
+        return self.SelectDict(pool_type=u"user", parameter=param, fields=fields, operators={self.identityField:u"IN"})
+    
+    
+    def GetUsersWithGroup(self, group, fields=None, activeOnly=True):
+        """
+        """
+        param = {u"groups":group}
+        if activeOnly:
+            param[u"pool_state"] = 1
+        operators = {u"groups": "LIKE"}
+        if not fields:
+            fields = [u"name",u"groups"]
+        elif not u"groups" in fields:
+            fields = list(fields)
+            fields.append(u"groups")
+        users = self.SelectDict(pool_type=u"user", parameter=param, fields=fields, operators=operators)
+        # verify groups
+        verified = []
+        for u in users:
+            try:
+                g = json.loads(u["groups"])
+            except:
+                continue
+            if group in g:
+                verified.append(u)
+        return verified
+
+
+    def DeleteUser(self, ident, currentUser=None):
+        """
+        returns status and report list
+        """
+        report = []
+        if isinstance(ident, basestring):
+            if not ident:
+                report.append(_(u"Invalid user."))
+                return False, report
+    
+            user = self.LookupUser(ident=ident, activeOnly=0)
+            if not user:
+                report.append(_(u"Invalid username."))
+                return False, report
+        else:
+            user = ident
+
+        if IAdminUser.providedBy(user):
+            report.append(_(u"You cannot delete the admin user."))
+            return False, report
+            
+        if not self.Delete(user.id, obj=user, user=currentUser):
+            report.append(_(u"Sorry. An error occured."))
+            return False, report
+
+        report.append(_(u"User deleted."))
+        return True, report
+
+
+    # to be removed ------------------------------------------------------
+    def GetUserGroups(self, name, activeOnly=1):
+        """
+        """
+        user = self.GetUser(name, activeOnly=activeOnly)
+        if not user:
+            return None
+        return user.data.groups
+
+
     def Encrypt(self, string):
         try:
             return base64.encodestring(string)
@@ -389,104 +441,6 @@ class root(UserCache, RootBase):
             return string
 
 
-    # User ------------------------------------------------------------------------------------------------------
-
-    def GetUserByName(self, name, activeOnly=1):
-        """
-        """
-        user = self.LookupUser(name=name, activeOnly=activeOnly)
-        if not user and self.app.configuration.get("loginByEmail"): 
-            user = self.GetUserByMail(name, activeOnly)
-        return user
-
-
-    def GetUserByID(self, id, activeOnly=1):
-        """
-        """
-        user = self.GetObj(id)
-        if not user or (activeOnly and not user.meta.get("pool_state")==1):
-            return None
-        return user
-
-
-    def GetUserByMail(self, email, activeOnly=1):
-        """
-        """
-        user = self.Select(pool_type=u"user", parameter={u"email":email}, fields=[u"id",u"name",u"email"], max=2, operators={u"email":u"="})
-        if len(user) != 1:
-            return None
-        if user[0][2] != email:
-            return None
-        return self.LookupUser(name=user[0][1], activeOnly=activeOnly)
-
-
-    def GetUserGroups(self, name, activeOnly=1):
-        """
-        """
-        user = self.GetUserByName(name, activeOnly=activeOnly)
-        if not user:
-            return None
-        return user.data.groups
-
-
-    def LookupUser(self, name=None, id=None, activeOnly=1, reloadFromDB=0):
-        """
-        """
-        if id:
-            user = self.GetObj(id)
-            if not user or (activeOnly and not user.meta.get("pool_state")==1):
-                return None
-            return user
-        elif name:
-            user = self.GetFromCache(name)
-            if user:
-                return user
-            # load admin user from configuration
-            try:
-                app = self.app
-                if app.configuration.admin["name"] == name:
-                    return adminuser(app.configuration.admin)
-            except:
-                pass
-            
-            user =  self.Select(pool_type=u"user", parameter={u"name":name}, fields=[u"id"], max=2, operators={u"name":u"="})
-            if len(user)==0:
-                return None
-            id = user[0][0]
-            if id:
-                user = self.GetObj(id)
-                if not user or (activeOnly and not user.meta.get("pool_state")==1):
-                    return None
-                self.Cache(user, name)
-                return user
-        return None
-
-
-    def GetUserInfos(self, userids, fields=["name", "email", "title"], activeOnly=True):
-        """
-        """
-        param = {u"name":userids}
-        if activeOnly:
-            param[u"pool_state"] = 1
-        users = self.SelectDict(pool_type=u"user", parameter=param, fields=fields, operators={u"name":u"IN"})
-        return users
-    
-    
-    def GetUsersWithGroup(self, group, fields=[u"name"], activeOnly=True):
-        """
-        """
-        param = {u"groups":group}
-        if activeOnly:
-            param[u"pool_state"] = 1
-        users = self.SelectDict(pool_type=u"user", parameter=param, fields=fields)
-        return users
-
-
-    def GetUsers(self, **kw):
-        """
-        """
-        users = self.SearchType(u"user", {u"pool_state":1}, [u"id",u"title",u"name",u"groups",u"lastlogin"])
-        return users
 
 
 # Root definition ------------------------------------------------------------------
@@ -504,66 +458,6 @@ configuration = RootConf(
 )
 
 
-
-class adminuser(object):
-    """
-    Admin User object with groups and login possibility. 
-    """
-    
-    def __init__(self, values):
-        self.id = 0
-        self.data = Conf(**values)
-        self.meta = Conf()
-        self.groups = self.data.groups = (u"group:admin",)
-
-    def __str__(self):
-        return self.data.get("name",str(self.id))
-
-    def Authenticate(self, password):
-        return password == self.data["password"]
-
-    
-    def Login(self):
-        """
-        events: login()
-        """
-        self.AddToCache()
-
-    def Logout(self):
-        """
-        events: logout()
-        """
-        self.RemoveFromCache()
-
-    def GetGroups(self, context=None):
-        """
-        groups
-        """
-        return self.groups
-
-    def InGroups(self, groups):
-        """
-        check if user has one of these groups
-        """
-        if isinstance(groups, basestring):
-            return groups in self.groups
-        for g in groups:
-            if g in self.groups:
-                return True
-        return False
-    
-    # System ------------------------------------------------
-
-    def AddToCache(self):
-        pass
-        #self.GetParent().Cache(self, self.id)
-
-    def RemoveFromCache(self):
-        pass
-        #self.GetParent().RemoveCache(self.id)
-
-
-
 from nive.components.reform.schema import Invalid
 from nive.components.reform.schema import Email
 
@@ -574,7 +468,7 @@ def UsernameValidator(node, value):
     """
     # lookup name in database
     r = node.widget.form.context.root()
-    u = r.Select(pool_type=u"user", parameter={u"name": value}, fields=[u"id",u"name",u"email"], max=2, operators={u"name":u"="})
+    u = r.LookupUser(name=value, activeOnly=0)
     if u:
         # check if its the current user
         ctx = node.widget.form.context
@@ -592,7 +486,7 @@ def EmailValidator(node, value):
     Email()(node, value)
     # lookup email in database
     r = node.widget.form.context.root()
-    u = r.Select(pool_type=u"user", parameter={u"email": value}, fields=[u"id",u"name",u"email"], max=2, operators={u"email":u"="})
+    u = r.LookupUser(email=value, activeOnly=0)
     if u:
         # check if its the current user
         ctx = node.widget.form.context
